/***************************************************************************
 *	vol_gui.c - {[fth
 *	
 *			1994/10/ 5 (Man.) -- 
 *
 *					Programmed by Nobuyuki Maruichi
 *	Note.
 *			SHIFT JIS Kanji Code / JIS X0208-1990
 ***************************************************************************/

/***********************************************
 *	{IȃCN[hEt@C
 ***********************************************/
#include <stdio.h>
#include <stdlib.h>

/***********************************************
 *	X Toolkit ̊{IȃCN[hEt@C
 ***********************************************/
#include <X11/Intrinsic.h>
#include <X11/StringDefs.h>
#include <X11/Shell.h>

/*****************************************
 * Atena Widget ̃CN[hEt@C
 *****************************************/
#include <X11/Xaw/Paned.h>
#include <X11/Xaw/Box.h>
#include <X11/Xaw/Form.h>
#include <X11/Xaw/Label.h>
#include <X11/Xaw/Command.h>
#include <X11/Xaw/Toggle.h>
#include <X11/Xaw/Repeater.h>
#include <X11/Xaw/Scrollbar.h>
#ifdef	X11R5
# include <X11/Xaw/Xawi18n.h>
# include <X11/Xaw/XawIm.h>
#endif	/*X11R5*/
/*******************************************
 *	{[Ex\p̃EBWFbg
 *******************************************/
#include "Level.h"

/**************************************
 * mwxcd p̃CN[hEt@C
 **************************************/
#include "mwxcd.h"
#include "vol_ioc.h"

typedef enum _volumeType {
	Monaural = 1, Stereo
} VolumeType;

typedef enum _targetType {
	MONO = -1, LEFT = 0, RIGHT
} TargetType;

typedef struct volume_widgets {
	Widget volumeArea;
	Widget volumeLabel;
	Widget volumeMeter;
	Widget volumeMin;
	Widget volumeMax;
	Widget volumeDown;
	Widget volumeUp;
	int monoVolum;
} MonauralInfo;

typedef struct stereo_volume_widgets {
	Widget volumeArea;
	Widget volumeLeftLabel;
	Widget volumeRightLabel;
	Widget volumeArea1;
	Widget volumeArea2;
	Widget volumeLeftMeter;
	Widget volumeRightMeter;
	Widget volumeSlider;
	Widget volumeMin;
	Widget volumeMax;
	Widget volumeSeparate;
	Widget volumeMute;
	Widget volumeDown;
	Widget volumeUp;
	int leftVolume;
	int rightVolume;
	int muteLeftLevel;
	int muteRightLevel;
} StereoInfo;

typedef struct _volumeInfo {
	int volumeType;
	union VolumeWidgets {
		MonauralInfo mono;
		StereoInfo stereo;
	} info;
} VolumeInfo;

typedef struct _volumeTargetInfo {
	String name;
	int target;
	int otherVolumeId;
	VolumeInfo **parts_ptr_ptr;
} VolumeTargetInfo;

extern Pixmap vup_pixmap, vdown_pixmap;
Widget pVolShell;

VolumeInfo *cdv, *linev, *mic;

static VolumeTargetInfo volumeTarget[] = {
	{ "Line Left",  LEFT, LineIn_Right, &linev},	/* Line in Left */
	{"Line Right", RIGHT,  LineIn_Left, &linev},	/* Line in Left */
	{   "CD Left",  LEFT,   CDDA_Right,   &cdv},	/* CDDA in Left */
	{  "CD Right", RIGHT,    CDDA_Left,   &cdv},	/* CDDA in Left */
	{       "MIC",  MONO,        MicIn,   &mic},	/*  Mic  */
	{     "MODEM",  MONO,     ModemOut,   NULL}		/* Modem */
};

typedef struct _mainVolumeInfo {
	int Leftid;
	int Rightid;
	const char *name;
} MainVolumeInfo;

static const MainVolumeInfo volumes[] = {
	{  CDDA_Left,   CDDA_Right, "CD"}, 
	{LineIn_Left, LineIn_Right, "Line"},
};

void TurnVolume(int volume_id, int level);

/***************************************************************************
 *	set_main_volume: CEEBhẼ{[ݒ肷B
 *	`
 *		void set_cd_volume(int volume_level, char *volume_string);
 *	
 *		int volume_level;		{[Exi|P`URj
 *		char *volume_string;	{[ExԂ
 *	
 *		
 ***************************************************************************/
void set_main_volume(char *volume_string, 
								const char *base_name, int vol1, int vol2)
{
	if(volume_string != NULL) {
		if(vol1 != vol2) {
				sprintf(volume_string, "%s L.R Separate", base_name);
		} else {
			if(vol1 >= 0) {
				sprintf(volume_string, "%s Volume%5.1fdB", 
						base_name, (vol1 - 63)/2.0);
				} else {
				sprintf(volume_string, "mute...(%s)    ", base_name);
			}
		}
	}
}

/***************************************************************************
 *	set_cd_volume: bcik^qj̃{[ݒ肷B
 *	`
 *		void set_cd_volume(int volume_level, char *volume_string);
 *	
 *		int volume_level;		{[Exi|P`URj
 *		char *volume_string;	{[ExԂ
 *	
 *		
 ***************************************************************************/
void set_cd_volume(int volume_level, char *volume_string)
{
	set_volume(CDDA_Left, volume_level);
	set_volume(CDDA_Right, volume_level);
	TurnVolume(CDDA_Left, volume_level);
	TurnVolume(CDDA_Right, volume_level);
	
	if(volume_string != NULL) {
		if(volume_level >= 0) {
			sprintf(volume_string, 
					"CD Volume%5.1fdB", (volume_level - 63)/2.0);
		} else {
			sprintf(volume_string, "mute...(CD)    ");
		}
	}
}

/***************************************************************************
 *	shift_cd_volume: bcik^qj{[ω
 *	`
 *		void shift_cd_volume(int d_volume, char *volume_string);
 *	
 *		int d_volume;			{[Ex̕ωl
 *		char *volume_string;	{[ExԂ
 *	
 *		
 ***************************************************************************/
int shift_cd_volume(int d_volume, char *volume_string)
{
	int status = FALSE;
	int volume_level;
	int leftVolume, rightVolume;
	
	leftVolume  = get_volume(CDDA_Left);
	rightVolume = get_volume(CDDA_Right);
	if(leftVolume == -2 || rightVolume == -2) {
		return(status);
	}
	volume_level = ((leftVolume == rightVolume)?
						(leftVolume):(leftVolume + rightVolume) / 2);
	volume_level += d_volume;
	
	if(volume_level >= -1 && volume_level <= 63) {
		set_cd_volume(volume_level, volume_string);
		status = TRUE;
	}
	return(status);
}

/***************************************************************************
 *	shift_main_volume: {[ω
 *	`
 *		void shift_cd_volume(int d_volume, char *volume_string);
 *	
 *		int d_volume;			{[Ex̕ωl
 *		char *volume_string;	{[ExԂ
 *	
 *		
 ***************************************************************************/
int shift_main_volume(int vDev, int dVol)
{
	int status = FALSE;
	int volume_level;
	int leftVolume, rightVolume;
	
	leftVolume  = get_volume(volumes[vDev].Leftid);
	rightVolume = get_volume(volumes[vDev].Rightid);
	if(leftVolume == -2 || rightVolume == -2) {
		return(status);
	}
	if(leftVolume >= rightVolume) {
		volume_level = ((dVol < 0)?(leftVolume):(rightVolume)) + dVol;
		if(volume_level >= -1 && volume_level <= 63) {
			set_volume(((dVol < 0)?
						(volumes[vDev].Leftid):(volumes[vDev].Rightid)), 
										volume_level);
			status = TRUE;
		}
	}
	if(rightVolume >= leftVolume) {
		volume_level = ((dVol < 0)?(rightVolume):(leftVolume)) + dVol;
		if(volume_level >= -1 && volume_level <= 63) {
			set_volume(((dVol < 0)?
						(volumes[vDev].Rightid):(volumes[vDev].Leftid)), 
										volume_level);
			status = TRUE;
		}
	}
	return(status);
}

/***************************************************************************
 *	set_line_volume: kik^qj̃{[ݒ肷B
 *	`
 *		void set_line_volume(int volume_level, char *volume_string);
 *	
 *		int volume_level;		{[Exi|P`URj
 *		char *volume_string;	{[ExԂ
 *	
 *		
 ***************************************************************************/
void set_line_volume(int volume_level, char *volume_string)
{
	set_volume(LineIn_Left, volume_level);
	set_volume(LineIn_Right, volume_level);
	TurnVolume(LineIn_Left, volume_level);
	TurnVolume(LineIn_Right, volume_level);
	
	if(volume_string != NULL) {
		if(volume_level >= 0) {
			sprintf(volume_string, 
					"Line Volume%5.1fdB", (volume_level - 63)/2.0);
		} else {
			sprintf(volume_string, "mute...(Line in)    ");
		}
	}
}

/***************************************************************************
 *	shift_line_volume: kik^qj{[ω
 *	`
 *		void shift_line_volume(int d_volume, char *volume_string);
 *	
 *		int d_volume;			{[Ex̕ωl
 *		char *volume_string;	{[ExԂ
 *	
 *		
 ***************************************************************************/
int shift_line_volume(int d_volume, char *volume_string)
{
	int status = FALSE;
	int volume_level;
	int leftVolume, rightVolume;
	
	leftVolume  = get_volume(LineIn_Left);
	rightVolume = get_volume(LineIn_Right);
	if(leftVolume == -2 || rightVolume == -2) {
		return(status);
	}
	volume_level = ((leftVolume == rightVolume)?
						(leftVolume):(leftVolume + rightVolume) / 2);
	volume_level += d_volume;
	
	if(volume_level >= -1 && volume_level <= 63) {
		set_line_volume(volume_level, volume_string);
		status = TRUE;
	}
	return(status);
}

/***************************************************************************
 *	ChangeVolumeLabel: CEEBhẼ{[Ex\
 *	`
 *		void ChangeVolumeLabel(void);
 *	
 *		
 ***************************************************************************/
void ChangeVolumeLabel(void)
{
	Dimension width;
	static char volume_string[20];
	extern Widget vol;
	extern int VolumeDevice;

	XtVaGetValues(vol, XtNwidth, &width, NULL);
	set_main_volume(volume_string, volumes[VolumeDevice].name, 
					get_volume(volumes[VolumeDevice].Leftid), 
					get_volume(volumes[VolumeDevice].Rightid));
	XtVaSetValues(vol, XtNlabel, volume_string, XtNwidth, width, NULL);
}

/***************************************************************************
 *	ChangeVolumeDev: yCD-DAz^yLine inz̃{[̐؂ւ
 *	`
 *		void ChangeVolumeDev(Widget w, int d_volume);
 *	
 *		Widget w;
 *		XtPointer client_data;
 *		XtPointer call_data;
 ***************************************************************************/
/* ARGSUSED */
void ChangeVolumeDev(Widget w, XtPointer client_data, XtPointer call_data)
{
	++VolumeDevice;
	VolumeDevice &= 0x01;
	ChangeVolumeLabel();
}

/***************************************************************************
 *	ChangeMainVolume: CEEBhẼ{[Exݒ^\
 *	`
 *		void ChangeVolume(int d_volume);
 *	
 *		int d_volume;
 ***************************************************************************/
/* ARGSUSED */
void ChangeMainVolume(int d_volume)
{
	extern int VolumeDevice;

	shift_main_volume(VolumeDevice, d_volume);
	ChangeVolumeLabel();

	TurnVolume( volumes[VolumeDevice].Leftid,  
							get_volume(volumes[VolumeDevice].Leftid));
	TurnVolume(volumes[VolumeDevice].Rightid, 
							get_volume(volumes[VolumeDevice].Rightid));
}

/***************************************************************************
 *	MainVolumeUp: CEEBhẼ{[Ex 0.5 dB グ
 *	`
 *		void MainVolumeUp(Widget w, 
 *						XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;
 *		XtPointer client_data;
 *		XtPointer call_data;
 ***************************************************************************/
void MainVolumeUp(Widget w, XtPointer client_data, XtPointer call_data)
{
	ChangeMainVolume(1);
}

/***************************************************************************
 *	VolumeDown: CEEBhẼ{[Ex 0.5 dB 
 *	`
 *		void VolumeDown(Widget w, 
 *							XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;
 *		XtPointer client_data;
 *		XtPointer call_data;
 ***************************************************************************/
void MainVolumeDown(Widget w, XtPointer client_data, XtPointer call_data)
{
	ChangeMainVolume(-1);
}

/***************************************************************************
 *	PopupVolume: {[EBhẼ|bvAbv
 *	`
 *		void PopupVolume(Widget w, 
 *							XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;	
 *		XtPointer client_data;	
 *		XtPointer call_data;	
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void PopupVolume(Widget w, XtPointer client_data, XtPointer call_data)
{
	PopdownInfo *popdownInfo = (PopdownInfo*)client_data;
    Widget popupButton = popdownInfo->popupWidget;
    Position x, y;
    Dimension width, height;

	/*********************************************************
	 *	|bvAbv̋NɂȂEBWFbƂ̑傫擾
	 *********************************************************/
	XtVaGetValues(popupButton, 
			XtNwidth, &width,
			XtNheight, &height,
			NULL);
	
	/***********************************
	 *	Root Window ̍Wnɕϊ
	 ***********************************/
	XtTranslateCoords(popupButton,		/* Widget */
			(Position) 0,				/* x */
			(Position) height/2,		/* y */
			&x, &y);					/* coords on root window */
	/*******************************************************************
	 *	|bvAbvVF̈ʒu肷i܂AoĂȂj
	 *******************************************************************/
	XtVaSetValues(pVolShell, 
			XtNx, x,
			XtNy, y,
			NULL);
}

/***************************************************************************
 *	TurnVolumeMeter: {[̕ύXɍăxEOt\XV
 *	`
 *		void TurnVolumeMeter(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		VolumeInfo *volume;		{[
 *		int target;				kEq܂́Am
 *		int level;				V{[Ex
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void TurnVolumeMeter(VolumeInfo *volume, int target, int level)
{
	int levelValue;
	Dimension iwidth, width;
	Widget targetWidget = NULL;

	switch(target) {
	case MONO:
		targetWidget = volume->info.mono.volumeMeter;
		break;
	case LEFT:
		targetWidget = volume->info.stereo.volumeLeftMeter;
		break;
	case RIGHT:
		targetWidget = volume->info.stereo.volumeRightMeter;
		break;
	}
	if(targetWidget != NULL) {
		XtVaGetValues(targetWidget, 
					XtNinternalWidth, &iwidth, XtNwidth, &width, NULL);
		levelValue = (level + 1) * (width - 2 * iwidth) / 64,
		XtVaSetValues(targetWidget, XtNlevelValue, levelValue, NULL);
	}
}

/***************************************************************************
 *	TurnVolumeMinMax: {[̕ύXɍčŏEő\XV
 *	`
 *		void TurnVolumeMinMax(VolumeInfo *volume, int target, int level);
 *	
 *		VolumeInfo *volume;		{[
 *		int target;				kEq܂́Am
 *		int level;				V{[Ex
 *	Ԃl
 *		
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void TurnVolumeMinMax(VolumeInfo *volume, int target, int level)
{
	Boolean minFlag, maxFlag;
	Widget minWidget = NULL;
	Widget maxWidget = NULL;

	switch(target) {
	case MONO:
		minWidget = volume->info.mono.volumeMin;
		maxWidget = volume->info.mono.volumeMax;
		level *= 2;
		break;
	case LEFT:
		minWidget = volume->info.stereo.volumeMin;
		maxWidget = volume->info.stereo.volumeMax;
		break;
	case RIGHT:
		minWidget = volume->info.stereo.volumeMin;
		maxWidget = volume->info.stereo.volumeMax;
		break;
	}
	minFlag = ((level == -1 * 2)?(TRUE):(FALSE));
	maxFlag = ((level == 63 * 2)?(TRUE):(FALSE));
	if(minWidget != NULL) {
		XtVaSetValues(minWidget, XtNstate, minFlag, NULL);
	}
	if(maxWidget != NULL) {
		XtVaSetValues(maxWidget, XtNstate, maxFlag, NULL);
	}
}

/***************************************************************************
 *	TurnVolumeLabel: {[̕ύXɍăfVx\XV
 *	`
 *		void TurnVolume(VolumeInfo *volume, 
 *						const char *base_name, int target, int level);
 *	
 *		VolumeInfo *volume;		{[
 *		const char *base_name;	{[
 *		int target;				kEq܂́Am
 *		int level;				V{[Ex
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void TurnVolumeLabel(VolumeInfo *volume,
					const char *base_name, int target, int level)
{
	char volume_string[30];
	Widget targetWidget = NULL;

	switch(target) {
	case MONO:
		targetWidget = volume->info.mono.volumeLabel;
		break;
	case LEFT:
		targetWidget = volume->info.stereo.volumeLeftLabel;
		break;
	case RIGHT:
		targetWidget = volume->info.stereo.volumeRightLabel;
		break;
	}
	if(targetWidget != NULL) {
		if(level >= 0)
			sprintf(volume_string, "%s %5.1fdB", base_name, (level - 63)/2.0);
		else
			sprintf(volume_string, "%s mute...",  base_name);
		XtVaSetValues(targetWidget, XtNlabel, volume_string, NULL);
	}
}

/***************************************************************************
 *	TurnVolume: {[̕ύXɍĕ\XV
 *	`
 *		void TurnVolume(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		int volume_id;			{[ʎq
 *		int level;				V{[Ex
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void TurnVolume(int volume_id, int level)
{
	Boolean muteFlag = FALSE;

	if(volumeTarget[volume_id].parts_ptr_ptr != NULL) {
		TurnVolumeMeter(*volumeTarget[volume_id].parts_ptr_ptr, 
								volumeTarget[volume_id].target, level);
		TurnVolumeLabel(*volumeTarget[volume_id].parts_ptr_ptr,
					volumeTarget[volume_id].name, 
					volumeTarget[volume_id].target, level);
		ChangeVolumeLabel();
		switch(volumeTarget[volume_id].target) {
		case MONO:
			TurnVolumeMinMax(*volumeTarget[volume_id].parts_ptr_ptr,
					volumeTarget[volume_id].target, level);
			break;
		case LEFT:
		case RIGHT:
			TurnVolumeMinMax(*volumeTarget[volume_id].parts_ptr_ptr,
				volumeTarget[volume_id].target, 
				level + get_volume(volumeTarget[volume_id].otherVolumeId));
			XtVaGetValues((*volumeTarget[volume_id].parts_ptr_ptr)->info.stereo.volumeMute, 
													XtNstate, &muteFlag, NULL);
			if(muteFlag == True) {
				XtVaSetValues((*volumeTarget[volume_id].parts_ptr_ptr)->info.stereo.volumeMute,
													XtNstate, False, NULL);
			}
			break;
		}
	}
}

/***************************************************************************
 *	TurnVolumes: XeIE{[̃xύXł̃xĐݒ^ĕ\
 *	`
 *		void TurnVolumes(int volume_id, int level);
 *	
 *		int volume_id;			{[ʎq
 *		int level;				V{[Ex
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void TurnVolumes(int volume_id, int level)
{
	Boolean separateFlag = FALSE;

	TurnVolume(volume_id, level);

	if((*volumeTarget[volume_id].parts_ptr_ptr)->volumeType == Stereo) {
		XtVaGetValues(
			(*volumeTarget[volume_id].parts_ptr_ptr)->info.stereo.volumeSeparate,
			XtNstate, &separateFlag, NULL);
	}
	if(separateFlag == FALSE) {
		set_volume(volumeTarget[volume_id].otherVolumeId, level);
		TurnVolume(volumeTarget[volume_id].otherVolumeId, level);
	}
}

/***************************************************************************
 *	VolumeMax: w肳ꂽ{[őɂ
 *	`
 *		void VolumeMax(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				gp
 *		XtPointer client_data;	{[ʎq
 *		XtPointer call_data;	gp
 *	Ԃl
 *		
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void VolumeMax(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volume_id = (int) client_data;
	
	set_volume(volume_id, 63);
	TurnVolume(volume_id, 63);
	set_volume(volumeTarget[volume_id].otherVolumeId, 63);
	TurnVolume(volumeTarget[volume_id].otherVolumeId, 63);
}

/***************************************************************************
 *	VolumeMin: w肳ꂽ{[ŏi~[gjɂ
 *	`
 *		void VolumeMin(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				gp
 *		XtPointer client_data;	{[ʎq
 *		XtPointer call_data;	gp
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void VolumeMin(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volume_id = (int) client_data;
	
	set_volume(volume_id, -1);
	TurnVolume(volume_id, -1);
	set_volume(volumeTarget[volume_id].otherVolumeId, -1);
	TurnVolume(volumeTarget[volume_id].otherVolumeId, -1);
}

/***************************************************************************
 *	VolumeMonoDown: w肳ꂽmE{[ -0.5dB _E
 *	`
 *		void VolumeMonoDown(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				gp
 *		XtPointer client_data;	{[ʎq
 *		XtPointer call_data;	gp
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void VolumeMonoDown(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	int level;
	
	level = get_volume(volId);
	if(level > -1) {
		set_volume(volId, level - 1);
		TurnVolume(volId, level - 1);
	}
}

/***************************************************************************
 *	VolumeMonoUp: w肳ꂽmE{[ -0.5 dB Abv
 *	`
 *		void VolumeMonoUp(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				gp
 *		XtPointer client_data;	{[ʎq
 *		XtPointer call_data;	gp
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void VolumeMonoUp(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	int level;
	
	level = get_volume(volId);
	if(level < 63) {
		set_volume(volId, level + 1);
		TurnVolume(volId, level + 1);
	}
}

/***************************************************************************
 *	VolumeStereoDown: w肳ꂽXeIE{[ -0.5 dB _E
 *	`
 *		void VolumeStereoDown(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				gp
 *		XtPointer client_data;	{[ʎq
 *		XtPointer call_data;	gp
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void VolumeStereoDown(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	int otherId;
	int level, olevel;
	
	otherId = volumeTarget[volId].otherVolumeId;
	level = get_volume(volId);
	olevel  = get_volume(otherId);
	if(olevel != level) {
		if(level > olevel) {
			set_volume(volId, level - 1);
			TurnVolume(volId, level - 1);
		} else {
			set_volume(otherId, olevel - 1);
			TurnVolume(otherId, olevel - 1);
		}
	} else {
		if(level > -1) {
			set_volume(volId, level - 1);
			TurnVolume(volId, level - 1);
			set_volume(otherId, olevel - 1);
			TurnVolume(otherId, olevel - 1);
		}
	}
}

/***************************************************************************
 *	VolumeStereoUp: w肳ꂽXeIE{[ +0.5 dB Abv
 *	`
 *		void VolumeStereoUp(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				gp
 *		XtPointer client_data;	{[ʎq
 *		XtPointer call_data;	gp
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void VolumeStereoUp(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	int otherId;
	int level, olevel;
	
	otherId = volumeTarget[volId].otherVolumeId;
	level = get_volume(volId);
	olevel  = get_volume(otherId);
	if(olevel != level) {
		if(level < olevel) {
			set_volume(volId, level + 1);
			TurnVolume(volId, level + 1);
		} else {
			set_volume(otherId, olevel + 1);
			TurnVolume(otherId, olevel + 1);
		}
	} else {
		if(level < 63) {
			set_volume(volId, level + 1);
			TurnVolume(volId, level + 1);
			set_volume(otherId, olevel + 1);
			TurnVolume(otherId, olevel + 1);
		}
	}
}

/***************************************************************************
 *	VolumeMute: w肳ꂽ{[ŏi~[gjɂ
 *	`
 *		void VolumeMute(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;	
 *		XtPointer client_data;	
 *		XtPointer call_data;	
 *	Ԃl
 *		
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void VolumeMute(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	int otherId;
	Boolean flag;
	StereoInfo *steroVolume;
	extern int VolumeDevice;
	
	otherId = volumeTarget[volId].otherVolumeId;

	steroVolume = &(*volumeTarget[volId].parts_ptr_ptr)->info.stereo;
	XtVaGetValues(w, XtNstate, &flag, NULL);
	if(flag == True) {
		steroVolume->muteLeftLevel  = get_volume(volId);
		steroVolume->muteRightLevel = get_volume(otherId);
		if(steroVolume->muteLeftLevel == -1 &&
				steroVolume->muteRightLevel == -1) {
			XtVaSetValues(w, XtNstate, False, NULL);
		}
		set_volume(volId, -1);
		set_volume(otherId, -1);
		if(volId == volumes[VolumeDevice].Leftid) {
			ChangeVolumeLabel();
		}
	} else {
		set_volume(  volId,  steroVolume->muteLeftLevel);
		set_volume(otherId, steroVolume->muteRightLevel);
		if(volId == volumes[VolumeDevice].Leftid) {
			ChangeVolumeLabel();
		}
		steroVolume->muteLeftLevel  = -1;
		steroVolume->muteRightLevel = -1;
	}
}

/***************************************************************************
 *	muteCut: w肳ꂽ{[̃~[gJbg
 *	`
 *		void muteCut(Widget w, XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;	
 *		XtPointer client_data;	
 *		XtPointer call_data;	
 *	Ԃl
 *		
 *	
 *		@~[gȌԂɖ߂
 ***************************************************************************/
/*ARGSUSED*/
void muteCut(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	int otherId;
	StereoInfo *steroVolume;
	
	steroVolume = &(*volumeTarget[volId].parts_ptr_ptr)->info.stereo;
	if(steroVolume->muteLeftLevel != -1 ||
				steroVolume->muteRightLevel != -1) {
		otherId = volumeTarget[volId].otherVolumeId;

		XtVaSetValues(steroVolume->volumeMute, XtNstate, False, NULL);
		set_volume(  volId,  steroVolume->muteLeftLevel);
		set_volume(otherId, steroVolume->muteRightLevel);
		steroVolume->muteLeftLevel  = -1;
		steroVolume->muteRightLevel = -1;
	}
}

/***************************************************************************
 *	muteCancel: w肳ꂽ{[̃~[gj
 *	`
 *		void muteCancel(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;	
 *		XtPointer client_data;	
 *		XtPointer call_data;	
 *	Ԃl
 *		
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
void muteCancel(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	StereoInfo *steroVolume;
	
	steroVolume = &(*volumeTarget[volId].parts_ptr_ptr)->info.stereo;
	if(steroVolume->muteLeftLevel != -1 ||
				steroVolume->muteRightLevel != -1) {
		XtVaSetValues(steroVolume->volumeMute, XtNstate, False, NULL);
		steroVolume->muteLeftLevel  = -1;
		steroVolume->muteRightLevel = -1;
	}
}

/***************************************************************************
 *	selectVolumeTarget: {[hc獶ȆΉ̂hc擾
 *	`
 *		int selectVolumeTarget(int volumeId, int target);
 *	
 *		int volumeId;	{[hc
 *		int target;		kEqw
 *	Ԃl
 *		EOF			G[ikEqȊOw肵Ƃj
 *		EOFȊO		kEq̎w肳ꂽ̃{[hc
 *	
 *		
 ***************************************************************************/
/*ARGSUSED*/
int selectVolumeTarget(int volumeId, int target)
{
	switch(volumeTarget[volumeId].target) {
	default:
		volumeId = EOF;
		break;
	case LEFT:
	case RIGHT:
		if(volumeTarget[volumeId].target != target) {
			volumeId = volumeTarget[volumeId].otherVolumeId;
		}
		break;
	}
	return(volumeId);
}

/***************************************************************************
 *	muteStereoCancel: w肳ꂽXeIE{[̃~[gj
 *	`
 *		void muteStereoCancel(Widget w, 
 *								XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				gp
 *		XtPointer client_data;	{[hcihms^ɃLXgj
 *		XtPointer call_data;	gp
 *	
 *		@w肳ꂽXeIE{[~[gԂłƂÃ~
 *		[g^jBkEqƗ䂩AłȂɂē삪
 *		قȂB
 *		
 *		PDkEqƗ
 *			@ݒ肳Ȃ̃{[~[gÃ{[
 *			ɖ߂B
 *		QDkEq
 *			@ɗ̃{[ݒ肳̂ŁA{[Ex
 *			ύXɖ̏Ԃ̂܂܂ɂĂB
 ***************************************************************************/
/*ARGSUSED*/
void muteStereoCancel(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volId = (int) client_data;
	int leftVolumeId, rightVolumeId;
	Boolean separateFlag = FALSE;
	StereoInfo *steroVolume;
	
	leftVolumeId  = selectVolumeTarget(volId,  LEFT);
	rightVolumeId = selectVolumeTarget(volId, RIGHT);
	steroVolume = &(*volumeTarget[volId].parts_ptr_ptr)->info.stereo;
	if(steroVolume->muteLeftLevel != -1 ||
				steroVolume->muteRightLevel != -1) {
		XtVaSetValues(steroVolume->volumeMute, XtNstate, False, NULL);
		XtVaGetValues(
			(*volumeTarget[volId].parts_ptr_ptr)->info.stereo.volumeSeparate,
											XtNstate, &separateFlag, NULL);
		if(separateFlag == True) {
			switch(volumeTarget[volId].target) {
			case RIGHT:
				set_volume( leftVolumeId,  steroVolume->muteLeftLevel);
				break;
			case LEFT:
				set_volume(rightVolumeId, steroVolume->muteRightLevel);
				break;
			}
		}
		steroVolume->muteLeftLevel  = -1;
		steroVolume->muteRightLevel = -1;
	}
}

/***************************************************************************
 *	e_volume_set: {[ݒ肷ilevel EBWFbg̃R[obNj
 *	`
 *		void e_volume_set(Widget w, 
 *							XtPointer client_data, XtPointer call_data);
 *	
 *		Widget w;				level EBWFbg
 *		XtPointer client_data;	{[ʎq
 *		XtPointer call_data;	x
 *	
 *
 ***************************************************************************/
void e_volume_set(Widget w, XtPointer client_data, XtPointer call_data)
{
	int volume_level = 0;
	int target = (int) client_data;
	XawLevelReturnStruct *levelData = (XawLevelReturnStruct*) call_data;

	if(levelData->maxValue > 0) {
		volume_level = 64 * levelData->levelValue / levelData->maxValue - 1;
		/*
		 *	printf("Volume Level %d (target %d)\n", volume_level, target);
		 */
		set_volume(target, volume_level);
		TurnVolumes(target, volume_level);
	}
}

/***************************************************************************
 *	volumeSeparateInit: 
 *	`
 *		void volumeSeparateInit(StereoInfo *stereo_pointer,
 *										int leftVolume, int rightVolume);
 *	
 *		StereoInfo *stereo_pointer;
 *		int leftVolume;	
 *		int rightVolume;	
 *	
 *
 ***************************************************************************/
void volumeSeparateInit(StereoInfo *stereo_pointer, 
										int leftVolume, int rightVolume)
{
	if(leftVolume != rightVolume) {
		XtVaSetValues(stereo_pointer->volumeSeparate, XtNstate, True, NULL);
	}
}

/***************************************************************************
 *	stereo_volume: XeIE{[̏s
 *	`
 *		void stereo_volume(Widget parent, Widget box);
 *	
 *		Widget parent;
 *	
 *
 ***************************************************************************/
VolumeInfo *stereo_volume(Widget parent, const char *baseName)
{
	char widgetName[80];
	VolumeInfo *alloc_pointer;
	StereoInfo *stereo_pointer;

	alloc_pointer = (VolumeInfo*)malloc(sizeof(VolumeInfo));
	stereo_pointer = &alloc_pointer->info.stereo;
	if(alloc_pointer != NULL) {
		alloc_pointer->volumeType = Stereo;
		sprintf(widgetName, "%sVolArea", baseName);
		stereo_pointer->volumeArea = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				panedWidgetClass,				/* widget class */
				parent,							/* parent widget */
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolArea1", baseName);
		stereo_pointer->volumeArea1 = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				panedWidgetClass,				/* widget class */
				stereo_pointer->volumeArea,		/* parent widget */
				XtNorientation, XtorientHorizontal, 
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolArea2", baseName);
		stereo_pointer->volumeArea2 = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				panedWidgetClass,				/* widget class */
				stereo_pointer->volumeArea,		/* parent widget */
				XtNorientation, XtorientHorizontal, 
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolLeftLabel", baseName);
		stereo_pointer->volumeLeftLabel = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				labelWidgetClass,				/* widget class */
				stereo_pointer->volumeArea1,		/* parent widget */
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolRightLabel", baseName);
		stereo_pointer->volumeRightLabel = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				labelWidgetClass,				/* widget class */
				stereo_pointer->volumeArea2,		/* parent widget */
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolLeftMeter", baseName);
		stereo_pointer->volumeLeftMeter = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				levelWidgetClass,				/* widget class */
				stereo_pointer->volumeArea1,	/* parent widget */
				XtNlevelMaxValue, 100, 
				XtNwidth, 110, 					/* Set width */
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolRightMeter", baseName);
		stereo_pointer->volumeRightMeter = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				levelWidgetClass,				/* widget class */
				stereo_pointer->volumeArea2,	/* parent widget */
				XtNlevelMaxValue, 100, 
				XtNwidth, 110, 					/* Set width */
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolMin", baseName);
		stereo_pointer->volumeMin = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				toggleWidgetClass,				/* widget class */
				stereo_pointer->volumeArea1,	/* parent widget */
				XtNlabel, "Min",				
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolMax", baseName);
		stereo_pointer->volumeMax = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				toggleWidgetClass,				/* widget class */
				stereo_pointer->volumeArea1,	/* parent widget */
				XtNlabel, "Max",				
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolDown", baseName);
		stereo_pointer->volumeDown = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				repeaterWidgetClass,			/* widget class */
				stereo_pointer->volumeArea2,	/* parent widget */
				XtNskipAdjust, True, 
#define	BITMAP_SET
#ifdef	BITMAP_SET
				XtNbitmap, vdown_pixmap,	
#endif	/*BITMAP_SET*/
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolUp", baseName);
		stereo_pointer->volumeUp = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				repeaterWidgetClass,			/* widget class */
				stereo_pointer->volumeArea2,	/* parent widget */
				XtNskipAdjust, True, 
#ifdef	BITMAP_SET
				XtNbitmap, vup_pixmap,	
#endif	/*BITMAP_SET*/
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolMute", baseName);
		stereo_pointer->volumeMute = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				toggleWidgetClass,				/* widget class */
				stereo_pointer->volumeArea1,		/* parent widget */
				XtNlabel, "Mute",				
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolLevel", baseName);
		stereo_pointer->volumeSeparate = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				toggleWidgetClass,				/* widget class */
				stereo_pointer->volumeArea2,		/* parent widget */
				XtNlabel, "Separate",				
				NULL							/* argument list */
				);
		stereo_pointer->muteLeftLevel = -1;
		stereo_pointer->muteRightLevel = -1;
	}
	return(alloc_pointer);
}

/***************************************************************************
 *	mono_volume: mE{[̏s
 *	`
 *		void mono_volume(Widget parent, Widget box);
 *	
 *		Widget parent;
 *	
 *
 ***************************************************************************/
VolumeInfo *mono_volume(Widget parent, const char *baseName)
{
	char widgetName[80];
	/*char *widgetName_ptr;*/
	VolumeInfo *alloc_pointer;
	MonauralInfo *monaural_pointer;

	alloc_pointer = (VolumeInfo*)malloc(sizeof(VolumeInfo));
	monaural_pointer = &alloc_pointer->info.mono;
	if(alloc_pointer != NULL) {
		alloc_pointer->volumeType = Monaural;
		sprintf(widgetName, "%sVolArea", baseName);
		monaural_pointer->volumeArea = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				panedWidgetClass,				/* widget class */
				parent,							/* parent widget */
				XtNorientation, XtorientHorizontal, 
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolLabel", baseName);
		monaural_pointer->volumeLabel = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				labelWidgetClass,				/* widget class */
				monaural_pointer->volumeArea,		/* parent widget */
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolMeter", baseName);
		monaural_pointer->volumeMeter = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				levelWidgetClass,				/* widget class */
				monaural_pointer->volumeArea,	/* parent widget */
				XtNlevelMaxValue, 100, 
				XtNwidth, 110, 					/* Set width */
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolDown", baseName);
		monaural_pointer->volumeDown = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				repeaterWidgetClass,			/* widget class */
				monaural_pointer->volumeArea,	/* parent widget */
				XtNskipAdjust, True, 
#ifdef	BITMAP_SET
				XtNbitmap, vdown_pixmap,	
#endif	/*BITMAP_SET*/
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolUp", baseName);
		monaural_pointer->volumeUp = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				repeaterWidgetClass,			/* widget class */
				monaural_pointer->volumeArea,	/* parent widget */
				XtNskipAdjust, True, 
#ifdef	BITMAP_SET
				XtNbitmap, vup_pixmap,	
#endif	/*BITMAP_SET*/
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolMin", baseName);
		monaural_pointer->volumeMin = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				toggleWidgetClass,				/* widget class */
				monaural_pointer->volumeArea,	/* parent widget */
				XtNlabel, "Min",				
				NULL							/* argument list */
				);
		sprintf(widgetName, "%sVolMax", baseName);
		monaural_pointer->volumeMax = XtVaCreateManagedWidget(
				widgetName,						/* Widget Name */
				toggleWidgetClass,				/* widget class */
				monaural_pointer->volumeArea,	/* parent widget */
				XtNlabel, "Max",				
				NULL							/* argument list */
				);
	}
	return(alloc_pointer);
}

/***************************************************************************
 *	volume_init: {[EVF̏s
 *	`
 *		void volume_init(Widget topLevel, Widget popupButton);
 *	
 *		Widget topLevel;
 *		Widget popupButton;
 *	
 *
 ***************************************************************************/
void volume_init(Widget topLevel, Widget popupButton)
{
    Widget vpane, buttonbox, volbox, popdownVolume;
	static PopdownInfo popdownInfo;
	extern Boolean volmue_enable_flag;

	pVolShell = XtCreatePopupShell(
					"pVolShell",
					topLevelShellWidgetClass,
					topLevel,
					NULL,
					0
					);
    /*
     *	mwxcd Volume Controler:
     */
    vpane = XtVaCreateManagedWidget("vpane", 
    				panedWidgetClass, pVolShell, NULL);

    buttonbox = XtVaCreateManagedWidget("buttonbox", 
    				boxWidgetClass, vpane, NULL);
    volbox = XtVaCreateManagedWidget("volbox", 
    				boxWidgetClass, vpane, NULL);

    popdownVolume = XtCreateManagedWidget("popdownVolume", 
    				commandWidgetClass, buttonbox, NULL, 0);

	cdv = stereo_volume(volbox, "cd");
	if(cdv != NULL) {
		volumeSeparateInit(&cdv->info.stereo, 
						get_volume(CDDA_Left), get_volume(CDDA_Right));
		TurnVolumes( CDDA_Left,  get_volume(CDDA_Left));
		TurnVolumes(CDDA_Right, get_volume(CDDA_Right));
		XtAddCallback(cdv->info.stereo.volumeLeftMeter, 
					XtNcallback, muteStereoCancel, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeLeftMeter, 
					XtNcallback, e_volume_set, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeRightMeter, 
					XtNcallback, muteStereoCancel, (XtPointer)CDDA_Right);
		XtAddCallback(cdv->info.stereo.volumeRightMeter, 
					XtNcallback, e_volume_set, (XtPointer)CDDA_Right);
		XtAddCallback(cdv->info.stereo.volumeMin, 
					XtNcallback, muteCancel, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeMin, 
					XtNcallback, VolumeMin, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeMax, 
					XtNcallback, muteCancel, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeMax, 
					XtNcallback, VolumeMax, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeDown, 
					XtNcallback, muteCut, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeDown, 
					XtNcallback, VolumeStereoDown, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeUp, 
					XtNcallback, muteCut, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeUp, 
					XtNcallback, VolumeStereoUp, (XtPointer)CDDA_Left);
		XtAddCallback(cdv->info.stereo.volumeMute, 
					XtNcallback, VolumeMute, (XtPointer)CDDA_Left);
	}

	linev = stereo_volume(volbox, "line");
	if(linev != NULL) {
		volumeSeparateInit(&linev->info.stereo, 
					get_volume(LineIn_Left), get_volume(LineIn_Right));
		TurnVolumes( LineIn_Left,  get_volume(LineIn_Left));
		TurnVolumes(LineIn_Right, get_volume(LineIn_Right));
		XtAddCallback(linev->info.stereo.volumeLeftMeter, 
					XtNcallback, muteStereoCancel, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeLeftMeter, 
					XtNcallback, e_volume_set, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeRightMeter, 
					XtNcallback, muteStereoCancel, (XtPointer)LineIn_Right);
		XtAddCallback(linev->info.stereo.volumeRightMeter, 
					XtNcallback, e_volume_set, (XtPointer)LineIn_Right);
		XtAddCallback(linev->info.stereo.volumeMin, 
					XtNcallback, muteCancel, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeMin, 
					XtNcallback, VolumeMin, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeMax, 
					XtNcallback, muteCancel, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeMax, 
					XtNcallback, VolumeMax, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeDown, 
					XtNcallback, muteCut, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeDown, 
					XtNcallback, VolumeStereoDown, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeUp, 
					XtNcallback, muteCut, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeUp, 
					XtNcallback, VolumeStereoUp, (XtPointer)LineIn_Left);
		XtAddCallback(linev->info.stereo.volumeMute, 
					XtNcallback, VolumeMute, (XtPointer)LineIn_Left);
	}
	mic = mono_volume(volbox, "mic");
	if(mic != NULL) {
		TurnVolumes( MicIn,  get_volume(MicIn));
		XtAddCallback(mic->info.mono.volumeMeter, 
					XtNcallback, e_volume_set, (XtPointer)MicIn);
		XtAddCallback(mic->info.mono.volumeMin, 
					XtNcallback, VolumeMin, (XtPointer)MicIn);
		XtAddCallback(mic->info.mono.volumeMax, 
					XtNcallback, VolumeMax, (XtPointer)MicIn);
		XtAddCallback(mic->info.mono.volumeDown, 
					XtNcallback, VolumeMonoDown, (XtPointer)MicIn);
		XtAddCallback(mic->info.mono.volumeUp, 
					XtNcallback, VolumeMonoUp, (XtPointer)MicIn);
	}
	
	popdownInfo.popupWidget = popupButton;
	popdownInfo.popdownShell = pVolShell;
	popdownInfo.client_data = NULL;
	XtAddCallback(popupButton, XtNcallback, PopupVolume, &popdownInfo);
	XtAddCallback(popupButton, XtNcallback, XtCallbackNone, pVolShell);
	XtAddCallback(popdownVolume, 
					XtNcallback, PopdownTargetShell, &popdownInfo);
	XtSetSensitive(popupButton, volmue_enable_flag);
}

